﻿using ProjectRepairWork.Entities;
using ProjectRepairWork.Repositories;
using ProjectRepairWork.Repositories.Implementation;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectRepairWork.Forms
{
    public partial class FormBuildingMaterial : Form
    {
        private readonly IBuildingMaterialsRepository _buildingMaterialsRepository;
        private int? _buildingMaterialsId;

        public int Id
        {
            set
            {
                try
                {
                    var buildingMaterials = _buildingMaterialsRepository.ReadBuildingMaterialsById(value);
                    if (buildingMaterials == null)
                    {
                        throw new InvalidDataException(nameof(buildingMaterials));
                    }
                    textBoxBuildingMaterial.Text = buildingMaterials.MaterialsType;
                    _buildingMaterialsId = value;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }
        public FormBuildingMaterial(IBuildingMaterialsRepository buildingMaterialsRepository)
        {
            InitializeComponent();
            _buildingMaterialsRepository = buildingMaterialsRepository ?? throw new ArgumentNullException(nameof(buildingMaterialsRepository));
        }
        private void Cancel_Click(object sender, EventArgs e) => Close();
        private BuildingMaterials CreateBuildingMaterials(int id) => BuildingMaterials.CreatEntity(
       id,
       textBoxBuildingMaterial.Text);

    
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxBuildingMaterial.Text))
                {
                    throw new Exception("Имеются незаполненные поля");
                }

                if (_buildingMaterialsId.HasValue)
                {
                    _buildingMaterialsRepository.UpdateBuildingMaterials(CreateBuildingMaterials(_buildingMaterialsId.Value));
                }
                else
                {
                    _buildingMaterialsRepository.CreateBuildingMaterials(CreateBuildingMaterials(0));
                }

                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        
        private void ButtonCancel_Click(object sender, EventArgs e) => Close();
        
    }
}
