﻿
using Microsoft.Extensions.Logging;
using ProjectRepairWork.Repositories;
using ProjectRepairWork.Repositories.Implementation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectRepairWork.Reports;

public class DocReport
{
    
    private readonly IContractorsRepository _contractorsRepository;
    private readonly IPremisesRepository _premisesRepository;
    private readonly IWorksRepository _worksRepository;
    private readonly ILogger<DocReport> _logger;

    public DocReport(ILogger<DocReport> logger, IContractorsRepository contractorsRepository, IPremisesRepository premisesRepository, IWorksRepository worksRepository)
    {
         
        _contractorsRepository = contractorsRepository ?? throw new ArgumentNullException(nameof(contractorsRepository));
        _premisesRepository = premisesRepository ?? throw new ArgumentNullException(nameof(premisesRepository));
        _worksRepository = worksRepository ?? throw new ArgumentNullException(nameof(worksRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateDoc(string filePath, bool includeContractors, bool includePremises, bool includeWork)
    {
        try
        {
            var builder = new WordBuilder(filePath).AddHeader("Документ со справочниками");


            if (includeContractors)
            {
                builder.AddParagraph("Подрядчики").AddTable([2400, 2400], GetContractors());
            }

            if (includePremises)
            {
                builder.AddParagraph("Помещения").AddTable([2400, 2400, 2400, 2400, 2400, 2400], GetPremises());
            }

            if (includeWork)
            {
                builder.AddParagraph("Работы").AddTable([2400, 2400], GetWork());
            }

            builder.Build();

            return true;
        }

        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<string[]> GetContractors()
    {
        return [
            ["Название компании", "Контакты"],
            .. _contractorsRepository
                .ReadContractors()
                .Select(x => new string[] { x.CompanyName, x.Contacts}),
        ];
    }

    private List<string[]> GetPremises()
    {
        return [
            ["Имя клиента", "Адрес", "Площадь", "Тип помещения","Дата начала","Дата конца"],
            .. _premisesRepository
                .ReadPremises()
                .Select(x => new string[] { x.ClientName, x.Address,  x.Area.ToString(),  x.PremisesType.ToString(),  x.DateStart.ToString(), x.DateEnd.ToString()}),
        ];
    }

    private List<string[]> GetWork()
    {
        return [
            ["Название работы", "Материалы"],
            .. _worksRepository
                .ReadWorks()
                .Select(x => new string[] { x.WorkName, x.MaterialsName.ToString()}),
        ];
    }
}
