﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectRepairWork.Entities;
using ProjectRepairWork.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectRepairWork.Repositories.Implementation;

public class ContractorsRepository : IContractorsRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<ContractorsRepository> _logger;
    public ContractorsRepository(IConnectionString connectionString, ILogger<ContractorsRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateContractors(Contractors contractors)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(contractors));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
INSERT INTO Contractors (companyName, Contacts)
VALUES (@CompanyName, @Contacts)";
            connection.Execute(queryInsert, contractors);
        }

        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void UpdateContractors(Contractors contractors)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
            JsonConvert.SerializeObject(contractors));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
                UPDATE Contractors
                SET
                CompanyName=@CompanyName,
                Contacts=@Contacts
                WHERE Id=@Id";
            connection.Execute(queryUpdate, contractors);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeletedContractors(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM Contractors
                WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public IEnumerable<Contractors> ReadContractors()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM Contractors";
            var contractors = connection.Query<Contractors>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(contractors));
            return contractors;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public Contractors ReadContractorsById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM Contractors
                WHERE Id=@id";
            var contractors = connection.QueryFirst<Contractors>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}",
                JsonConvert.SerializeObject(contractors));
            return contractors;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

   
}
