﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectRepairWork.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace ProjectRepairWork.Repositories.Implementation;

public class RepairRepository : IRepairRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<RepairRepository> _logger;

    public RepairRepository(IConnectionString connectionString, ILogger<RepairRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CraeteRepair(Repair repair)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(repair));
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryInsert = @"
                INSERT INTO Repair (ContractorsId, PremisesId, DateRepair)
                VALUES (@ContractorsId, @PremisesId, @DateRepair)
                RETURNING Id;";
            var repairId =
            connection.QueryFirst<int>(queryInsert, repair, transaction);
            var querySubInsert = @"
                INSERT INTO RepairRepair (RepairId, WorkId, Count)
                VALUES (@RepairId, @WorkId, @Count)";
            foreach (var elem in repair.RepairRepairList)
            {
                connection.Execute(querySubInsert, new
                {
                    RepairId = repairId,
                    elem.WorkId,
                    elem.Count
                }, transaction);
            }
            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public IEnumerable<Repair> ReadRepair(DateTime? dateForm = null, DateTime? dateTo = null, int? contractorsId = null, int? costId = null, int? worksId = null, int? premisesId = null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            var builder = new QueryBuilder();
            if (dateForm.HasValue)
            {
                builder.AddCondition("r.DateRepair >= @dateForm");
            }
            if (dateTo.HasValue)
            {
                builder.AddCondition("r.DateRepair <= @dateTo");
            }
            if (contractorsId.HasValue)
            {
                builder.AddCondition("r.ContractorsId = @contractorsId");
            }
            if (costId.HasValue)
            {
                builder.AddCondition("r.Cost = @costId");
            }
            if (worksId.HasValue)
            {
                builder.AddCondition("r.WorkId = @worksId");
            }
            if (premisesId.HasValue)
            {
                builder.AddCondition("r.PremisesId = @premisesId");
            }


            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = $@" 
                SELECT
                    r.*,
                    c.CompanyName as CompanyName,
                    p.PremisesType as PremisesType
                    
                FROM Repair r
                LEFT JOIN Contractors c on c.Id = r.ContractorsId
                LEFT JOIN RepairRepair rr on rr.Id = r.Id
                
                LEFT JOIN Premises p on p.Id = r.PremisesId
                {builder.Build()}";
            var repair = connection.Query<Repair>(querySelect, new { dateForm, dateTo });
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(repair));
            return repair;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public IEnumerable<RepairRepair> ReadRepairRepair()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM RepairRepair";
            var repairRepairs = connection.Query<RepairRepair>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(repairRepairs));
            return repairRepairs;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
